import pytest
from skidl import *
from .setup_teardown import *

def test_bus_1():
    # Make a bus.
    bus = Bus('BB', 11, 5)
    assert len(bus) == 16


def test_bus_2():
    # Make a bus from nets.
    gnd = Net('GND')
    a = Net('A')
    b = Net('B')
    c = Net()
    bus = Bus('BB', a, b, c)
    assert len(bus) == 3
    assert len(default_circuit.get_nets()) == 0


def test_bus_3():
    # Make a bus from ints and nets.
    gnd = Net('GND')
    a = Net('A')
    b = Net('B')
    c = Net()
    bus = Bus('BB', a, 5, b, c)
    assert len(bus) == 8
    assert len(default_circuit.get_nets()) == 0


def test_bus_4():
    # Make a bus from ints, nets and buses.
    gnd = Net('GND')
    a = Net('A')
    b = Net('B')
    c = Net()
    bus1 = Bus('AA', a, c, 2)
    assert len(bus1) == 4
    bus1.name = 'AA'
    assert bus1.name == 'AA'
    bus2 = Bus('BB', b, bus1, 3)
    assert len(bus2) == len(bus1) + 4
    assert len(default_circuit.get_nets()) == 0


def test_bus_5():
    # Connect a pin to a bus line.
    gnd = Net('GND')
    a = Net('A')
    b = Net('B')
    c = Net()
    bus = Bus('AA', a, a, 2)
    assert len(bus) == 4
    p = Pin()
    a += p
    assert isinstance(bus[0], Net)
    bus[0] += p
    assert len(a) == 1
    assert len(bus[0]) == 1
    assert len(bus[1]) == 1
    assert len(default_circuit.get_nets()) == 1


def test_bus_6():
    # Check bus line names.
    bus_prefix = 'AA'
    net = Net()
    pin = Pin()
    bus = Bus(bus_prefix, 4, net, 3, pin, 4)
    assert len(bus) == 13
    for i, n in enumerate(bus.get_nets()):
        assert bus_prefix + str(i) == n.name
    net.name = 'a'
    for i, n in enumerate(bus.get_nets()):
        if i == 4:
            assert net.name == n.name
        else:
            assert bus_prefix + str(i) == n.name


def test_bus_7():
    # Test bus connections.
    net = Net()
    pin = Pin()
    bus = Bus('AA', 2)
    net += pin
    net += bus[1]
    assert len(net) == 1


def test_bus_8():
    bus1 = Bus('A', 8)
    assert isinstance(bus1, Bus)
    nets = [Net() for _ in bus1.nets]
    for n in nets:
        n += Pin()
    bus1 += nets
    assert isinstance(bus1, Bus)
    bus2 = Bus('B', 8)
    assert isinstance(bus2, Bus)
    bus1 += bus2
    assert len(default_circuit.get_nets()) == len(bus1)


def test_bus_9():
    bus1 = Bus('A', 8)
    bus2 = Bus('B', 8)
    bus2 += 8 * Pin()
    bus1 += bus2[7:0]
    assert len(default_circuit.get_nets()) == len(bus1)


def test_bus_10():
    bus1 = Bus('A', 8)
    bus2 = Bus('B', 9)
    with pytest.raises(Exception):
        bus1 += bus2


def test_bus_11():
    bus1 = Bus('A', 8)
    bus2 = Bus('B', 9)
    with pytest.raises(Exception):
        bus2 += bus1


def test_bus_12():
    bus1 = Bus('A', 8)
    with pytest.raises(Exception):
        bus1[5:0] = 6 * Pin()


def test_bus_13():
    bus1 = Bus('A', 8)
    bus1 += 8 * Pin()
    with pytest.raises(Exception):
        bus1[5:0] = 6 * Pin()

def test_bus_14():
    bus1 = Bus('A',8)
    p1, p2 = 2 * Pin()
    p2 += bus1[5]
    p1 += p2
    p3 = Pin()
    bus1[5] += p3

def test_bus_15():
    bus1 = Bus('A', 8)
    nets = bus1[:][::-1]
    for n, i in zip(nets, range(8)[::-1]):
        assert 'A'+str(i) == n.name

def test_bus_copy_1():
    bus1 = Bus('A',8)
    bus2 = bus1()
    assert len(bus1) == len(bus2)



import pytest
from skidl import *
from .setup_teardown import *

def test_subcircuit_1():

    class Resistor(Part):
        def __init__(self, value, ref=None, footprint='Resistors_SMD:R_0805'):
            super().__init__('device', 'R', value=value, ref=ref, footprint=footprint)

    @subcircuit
    def resdiv():
        gnd = Net('GND') # Ground reference.
        vin = Net('VI')  # Input voltage to the divider.
        vout = Net('VO')  # Output voltage from the divider.

        r1 = Resistor('1k')
        r2 = Resistor('500')

        cap = Part('device','C', dest=TEMPLATE)
        c1 = cap()
        c2 = cap(value='1uF')

        bus1 = Bus('BB',10)

        r1[1] += vin  # Connect the input to the first resistor.
        r2[2] += gnd  # Connect the second resistor to ground.
        vout += r1[2], r2[1]  # Output comes from the connection of the two resistors.

    resdiv()
    resdiv()

    assert len(default_circuit.parts) == 8
    assert len(default_circuit.get_nets()) == 6
    assert len(default_circuit.buses) == 2

    ERC()
    generate_netlist()
    generate_xml()

def test_subcircuit_2():

    class Resistor(Part):
        def __init__(self, value, ref=None, footprint='Resistors_SMD:R_0805'):
            super().__init__('device', 'R', value=value, ref=ref, footprint=footprint)

    @subcircuit    
    def resdiv_1():
        gnd = Net('GND') # Ground reference.
        vin = Net('VI')  # Input voltage to the divider.
        vout = Net('VO')  # Output voltage from the divider.

        r1 = Resistor('1k')
        r2 = Resistor('500')

        cap = Part('device','C', dest=TEMPLATE)
        c1 = cap()
        c2 = cap(value='1uF')

        bus1 = Bus('BB',10)

        r1[1] += vin  # Connect the input to the first resistor.
        r2[2] += gnd  # Connect the second resistor to ground.
        vout += r1[2], r2[1]  # Output comes from the connection of the two resistors.

    @subcircuit    
    def resdiv_2():
        resdiv_1()
        resdiv_1()

        a = Net('GND') # Ground reference.
        b = Net('VI')  # Input voltage to the divider.
        c = Net('VO')  # Output voltage from the divider.

        r1 = Resistor('1k')
        r2 = Resistor('500')

        cap = Part('device','C', dest=TEMPLATE)
        c1 = cap()
        c2 = cap(value='1uF')

        bus1 = Bus('BB',10)

        r1[1] += a  # Connect the input to the first resistor.
        r2[2] += b  # Connect the second resistor to ground.
        c += r1[2], r2[1]  # Output comes from the connection of the two resistors.

    resdiv_2()

    assert len(default_circuit.parts) == 12
    assert len(default_circuit.get_nets()) == 9
    assert len(default_circuit.buses) == 3

    ERC()
    generate_netlist()
    generate_xml()



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

onsemi = SchLib(tool=SKIDL).add_parts(*[
        Part(name='CM1213A-01SO',dest=TEMPLATE,tool=SKIDL,keywords='ESD Protection diodes transient suppressor',description='Single Channel ESD Protection Array',ref_prefix='D',num_units=1,fplist=['SOT-23*'],do_erc=True,pins=[
            Pin(num='1',name='CH1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='VP',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='VN',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='NCP45560',dest=TEMPLATE,tool=SKIDL,keywords='load switch',description='- Controlled Load Switch with Low Ron',ref_prefix='U',num_units=1,fplist=['TO263-5'],do_erc=True,aliases=['NCP45560H', 'NCP45560L'],pins=[
            Pin(num='1',name='VIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='EN',do_erc=True),
            Pin(num='3',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='GND',do_erc=True),
            Pin(num='5',name='SR',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='PG',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='7',name='BLEED',do_erc=True),
            Pin(num='8',name='VOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='VOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='VOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='VOUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='VOUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='13',name='VIN',func=Pin.PWRIN,do_erc=True)]),
        Part(name='NUP2202',dest=TEMPLATE,tool=SKIDL,keywords='ESD Protection diodes transient suppressor',description='Transient voltage suppressor designed to protect high speed data lines from ESD, EFT, and lightning',ref_prefix='U',num_units=1,fplist=['*SC-70*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='~',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='NUP4202',dest=TEMPLATE,tool=SKIDL,keywords='ESD Protection diodes transient suppressor',description='Transient voltage suppressor designed to protect high speed data lines from ESD, EFT, and lightning',ref_prefix='U',num_units=1,fplist=['*SC-70*'],do_erc=True,pins=[
            Pin(num='1',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='~',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.PASSIVE,do_erc=True)])])


# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2018 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
Handles buses.
"""

from __future__ import unicode_literals
from __future__ import print_function
from __future__ import division
from __future__ import absolute_import

from builtins import range
from builtins import str
from future import standard_library
standard_library.install_aliases()

try:
    import __builtin__ as builtins
except ImportError:
    import builtins

from .defines import *
from .utilities import *


class Bus(object):
    """
    This class collects one or more nets into a group that can be indexed.

    Args:
        name: A string with the name of the bus.
        args: A list of ints, pins, nets, buses to attach to the net.

    Keyword Args:
        attribs: A dictionary of attributes and values to attach to
            the Net object.

    Example:
        ::

            n = Net()
            led1 = Part('device', 'LED')
            b = Bus('B', 8, n, led1['K'])
    """

    def __init__(self, name, *args, **attribs):

        # Define the member storing the nets so it's present, but it starts empty.
        self.nets = []

        # For Bus objects, the circuit object the bus is a member of is passed
        # in with all the other attributes. If a circuit object isn't provided,
        # then the default circuit object is added to the attributes.
        attribs['circuit'] = attribs.get('circuit', default_circuit)  # pylint: disable=undefined-variable

        # Attach additional attributes to the bus. (The Circuit object also gets
        # set here.)
        for k, v in attribs.items():
            setattr(self, k, v)

        # The bus name is set after the circuit is assigned so the name can be
        # checked against the other bus names already in that circuit.
        self.name = name

        # Add the bus to the circuit.
        self.circuit = None  # Bus won't get added if it's already seen as part of circuit.
        attribs[
            'circuit'] += self  # Add bus to circuit. This also sets self.circuit again.

        # Build the bus from net widths, existing nets, nets of pins, other buses.
        self.extend(args)

    def extend(self, *objects):
        """Extend bus by appending objects to the end (MSB)."""
        self.insert(len(self.nets), objects)

    def insert(self, index, *objects):
        """Insert objects into bus starting at indexed position."""

        from .Net import Net
        from .Bus import Bus
        from .Pin import Pin

        for obj in flatten(objects):
            if isinstance(obj, int):
                # Add a number of new nets to the bus.
                for _ in range(obj):
                    self.nets.insert(index, Net())
                index += obj
            elif isinstance(obj, Net):
                # Add an existing net to the bus.
                self.nets.insert(index, obj)
                index += 1
            elif isinstance(obj, Pin):
                # Add a pin to the bus.
                try:
                    # Add the pin's net to the bus.
                    self.nets.insert(index, obj.get_nets()[0])
                except IndexError:
                    # OK, the pin wasn't already connected to a net,
                    # so create a new net, add it to the bus, and
                    # connect the pin to it.
                    n = Net()
                    n += obj
                    self.nets.insert(index, n)
                index += 1
            elif isinstance(obj, Bus):
                # Add an existing bus to this bus.
                for n in reversed(obj.nets):
                    self.nets.insert(index, n)
                index += len(obj)
            else:
                logger.error(
                    'Adding illegal type of object ({}) to Bus {}.'.format(
                        type(obj), self.name))
                raise Exception

        # Assign names to all the unnamed nets in the bus.
        for i, net in enumerate(self.nets):
            if net.is_implicit():
                # Net names are the bus name with the index appended.
                net.name = self.name + str(i)

    def get_nets(self):
        """Return the list of nets contained in this bus."""
        return to_list(self.nets)

    def get_pins(self):
        """It's an error to get the list of pins attached to all bus lines."""
        logger.error("Can't get the list of pins on a bus!")
        raise Exception

    def copy(self, num_copies=1, **attribs):
        """
        Make zero or more copies of this bus.

        Args:
            num_copies: Number of copies to make of this bus.

        Keyword Args:
            attribs: Name/value pairs for setting attributes for the copy.

        Returns:
            A list of Bus copies or a Bus if num_copies==1.

        Raises:
            Exception if the requested number of copies is a non-integer or negative.

        Notes:
            An instance of a bus can be copied just by calling it like so::

                b = Bus('A', 8)  # Create a bus.
                b_copy = b(2)   # Get two copies of the bus.

            You can also use the multiplication operator to make copies::

                b = 10 * Bus('A', 8)  # Create an array of buses.
        """

        # Check that a valid number of copies is requested.
        if not isinstance(num_copies, int):
            logger.error(
                "Can't make a non-integer number ({}) of copies of a bus!".
                format(num_copies))
            raise Exception
        if num_copies < 0:
            logger.error(
                "Can't make a negative number ({}) of copies of a bus!".format(
                    num_copies))
            raise Exception

        copies = []
        for i in range(num_copies):

            cpy = Bus(self.name, self)

            # Attach additional attributes to the bus.
            for k, v in attribs.items():
                if isinstance(v, (list, tuple)):
                    try:
                        v = v[i]
                    except IndexError:
                        logger.error(
                            "{} copies of bus {} were requested, but too few elements in attribute {}!".
                            format(num_copies, self.name, k))
                        raise Exception
                setattr(cpy, k, v)

            copies.append(cpy)

        return list_or_scalar(copies)

    # Make copies with the multiplication operator or by calling the object.
    __mul__ = copy
    __rmul__ = copy
    __call__ = copy

    def __getitem__(self, *ids):
        """
        Return a bus made up of the nets at the given indices.

        Args:
            ids: A list of indices of bus lines. These can be individual
                numbers, net names, nested lists, or slices.

        Returns:
            A bus if the indices are valid, otherwise None.
        """

        from .NetPinList import NetPinList

        # Use the indices to get the nets from the bus.
        nets = []
        for ident in expand_indices(0, len(self) - 1, ids):
            if isinstance(ident, int):
                nets.append(self.nets[ident])
            elif isinstance(ident, basestring):
                nets.extend(filter_list(self.nets, name=ident))
            else:
                logger.error("Can't index bus with a {}.".format(type(ident)))
                raise Exception

        if len(nets) == 0:
            # No nets were selected from the bus, so return None.
            return None

        if len(nets) == 1:
            # Just one net selected, so return the Net object.
            return nets[0]

        # Multiple nets selected, so return them as a NetPinList list.
        return NetPinList(nets)

    def __setitem__(self, ids, *pins_nets_buses):
        """
        You can't assign to bus lines. You must use the += operator.

        This method is a work-around that allows the use of the += for making
        connections to bus lines while prohibiting direct assignment. Python
        processes something like my_bus[7:0] += 8 * Pin() as follows::

            1. Part.__getitem__ is called with '7:0' as the index. This
               returns a NetPinList of eight nets from my_bus.
            2. The NetPinList.__iadd__ method is passed the NetPinList and
               the thing to connect to the it (eight pins in this case). This
               method makes the actual connection to the part pin or pins. Then
               it creates an iadd_flag attribute in the object it returns.
            3. Finally, Bus.__setitem__ is called. If the iadd_flag attribute
               is true in the passed argument, then __setitem__ was entered
               as part of processing the += operator. If there is no
               iadd_flag attribute, then __setitem__ was entered as a result
               of using a direct assignment, which is not allowed.
        """

        # If the iadd_flag is set, then it's OK that we got
        # here and don't issue an error. Also, delete the flag.
        if getattr(pins_nets_buses[0], 'iadd_flag', False):
            del pins_nets_buses[0].iadd_flag
            return

        # No iadd_flag or it wasn't set. This means a direct assignment
        # was made to the pin, which is not allowed.
        logger.error("Can't assign to a bus! Use the += operator.")
        raise Exception

    def is_movable(self):
        """
        Return true if the bus is movable to another circuit.

        A bus  is movable if all the nets in it are movable.
        """
        for n in self.nets:
            if not n.is_movable():
                # One net not movable means the entire Bus is not movable.
                return False
        return True  # All the nets were movable.

    def connect(self, *pins_nets_buses):
        """
        Return the bus after connecting one or more nets, pins, or buses.

        Args:
            pins_nets_buses: One or more Pin, Net or Bus objects or
                lists/tuples of them.

        Returns:
            The updated bus with the new connections.

        Notes:
            You can connect nets or pins to a bus like so::

                p = Pin()       # Create a pin.
                n = Net()       # Create a net.
                b = Bus('B', 2) # Create a two-wire bus.
                b += p,n        # Connect pin and net to B[0] and B[1].
        """

        from .NetPinList import NetPinList

        nets = NetPinList(self.nets)
        nets += pins_nets_buses
        return self

    __iadd__ = connect

    @property
    def name(self):
        """
        Get, set and delete the name of the bus.

        When setting the bus name, if another bus with the same name
        is found, the name for this bus is adjusted to make it unique.
        """
        return self._name

    @name.setter
    def name(self, name):
        from .defines import BUS_PREFIX

        # Remove the existing name so it doesn't cause a collision if the
        # object is renamed with its existing name.
        self._name = None

        # Now name the object with the given name or some variation
        # of it that doesn't collide with anything else in the list.
        self._name = get_unique_name(self.circuit.buses, 'name',
                                     BUS_PREFIX, name)

    @name.deleter
    def name(self):
        """Delete the bus name."""
        del self._name

    def __str__(self):
        """Return a list of the nets in this bus as a string."""
        return self.name + ':\n\t' + '\n\t'.join(
            [n.__str__() for n in self.nets])

    __repr__ = __str__

    def __len__(self):
        """Return the number of nets in this bus."""
        return len(self.nets)



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

analog_devices = SchLib(tool=SKIDL).add_parts(*[
        Part(name='AD623AN',dest=TEMPLATE,tool=SKIDL,keywords='ad623 instumentation amplifier dip-8',description='Single Supply, Rail to Rail, Instumentation Amplifier, RoHS, DIP-8',ref_prefix='U',num_units=1,fplist=['DIP-8*'],do_erc=True,aliases=['AD623BN', 'AD623ANZ', 'AD623BNZ'],pins=[
            Pin(num='1',name='Rg',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',do_erc=True),
            Pin(num='3',name='+',do_erc=True),
            Pin(num='4',name='Vs-',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='Ref',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Vs+',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='Rg',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='AD623AR',dest=TEMPLATE,tool=SKIDL,keywords='ad623 instumentation amplifier soic-8',description='Single Supply, Rail to Rail, Instumentation Amplifier, RoHS, SOIC-8',ref_prefix='U',num_units=1,fplist=['SOIC-8*'],do_erc=True,aliases=['AD623ARZ', 'AD623BR', 'AD623BRZ'],pins=[
            Pin(num='1',name='Rg',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',do_erc=True),
            Pin(num='3',name='+',do_erc=True),
            Pin(num='4',name='Vs-',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='Ref',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Vs+',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='Rg',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='AD623ARM',dest=TEMPLATE,tool=SKIDL,keywords='ad623 instumentation amplifier msop-8',description='Single Supply, Rail to Rail, Instumentation Amplifier, RoHS, MSOP-8',ref_prefix='U',num_units=1,fplist=['MSOP-8*'],do_erc=True,aliases=['AD623ARMZ'],pins=[
            Pin(num='1',name='Rg',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='-',do_erc=True),
            Pin(num='3',name='+',do_erc=True),
            Pin(num='4',name='Vs-',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='Ref',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='Vs+',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='Rg',func=Pin.PASSIVE,do_erc=True)]),
        Part(name='AD8422ARMZ',dest=TEMPLATE,tool=SKIDL,keywords='ad8422 instumentation amplifier msop-8',description='Low Power, Rail to Rail, Instumentation Amplifier, MSOP-8',ref_prefix='U',num_units=1,fplist=['MSOP-8*'],do_erc=True,aliases=['AD8422BRMZ', 'AD8421ARMZ', 'AD8421BRMZ', 'AD8236ARMZ'],pins=[
            Pin(num='1',name='-',do_erc=True),
            Pin(num='2',name='Rg',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='Rg',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='+',do_erc=True),
            Pin(num='5',name='Vs-',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='Ref',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='Vs+',func=Pin.PWRIN,do_erc=True)]),
        Part(name='AD8422ARZ',dest=TEMPLATE,tool=SKIDL,keywords='ad8429 instumentation amplifier soic-8',description='Low Noise, Instumentation Amplifier, SOIC-8',ref_prefix='U',num_units=1,fplist=['SOIC-8*'],do_erc=True,aliases=['AD8422BRZ', 'AD8421ARZ', 'AD8421BRZ', 'AD8429ARZ', 'AD8429BRZ'],pins=[
            Pin(num='1',name='-',do_erc=True),
            Pin(num='2',name='Rg',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='Rg',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='+',do_erc=True),
            Pin(num='5',name='Vs-',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='Ref',func=Pin.PASSIVE,do_erc=True),
            Pin(num='7',name='~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='Vs+',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ADE7758',dest=TEMPLATE,tool=SKIDL,keywords='Energy Metering',description='Poly Phase Multifunction Energy Metering, SO-24',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x15.4mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='APCF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='DVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='IAP',do_erc=True),
            Pin(num='6',name='IAN',do_erc=True),
            Pin(num='7',name='IBP',do_erc=True),
            Pin(num='8',name='IBN',do_erc=True),
            Pin(num='9',name='ICP',do_erc=True),
            Pin(num='10',name='ICN',do_erc=True),
            Pin(num='20',name='CLKOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='11',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='~CS',do_erc=True),
            Pin(num='12',name='REFIN/OUT',do_erc=True),
            Pin(num='22',name='DIN',do_erc=True),
            Pin(num='13',name='VN',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='SCLK',do_erc=True),
            Pin(num='14',name='VCP',do_erc=True),
            Pin(num='24',name='DOUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='VBP',do_erc=True),
            Pin(num='16',name='VAP',do_erc=True),
            Pin(num='17',name='VARCF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='~IRQ',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='19',name='CLKIN',do_erc=True)]),
        Part(name='ADM2484E',dest=TEMPLATE,tool=SKIDL,keywords='RS485 Transceiver RS422 Transceiver',description='Isolated RS485/RS422 Transceiver, Half-/Full-Duplex, 500kbps,SOIC-16W',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x10.3mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='RxD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='~RE',do_erc=True),
            Pin(num='5',name='DE',do_erc=True),
            Pin(num='6',name='TxD',do_erc=True),
            Pin(num='7',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='8',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='11',name='Y',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='Z',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='B',do_erc=True),
            Pin(num='14',name='A',do_erc=True),
            Pin(num='15',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD2',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ADM2587E',dest=TEMPLATE,tool=SKIDL,keywords='RS485 Transciever,RS422 Transciever',description='Isolated RS485/RS422 Transciever,Integrated Isolated DC-DC Converter, 500kbps,SO-20',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x12.8mm*Pitch1.27mm*'],do_erc=True,aliases=['ADM2582E'],pins=[
            Pin(num='1',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='RxD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='~RE',do_erc=True),
            Pin(num='6',name='DE',do_erc=True),
            Pin(num='7',name='TxD',do_erc=True),
            Pin(num='8',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='Visoout',func=Pin.PWROUT,do_erc=True),
            Pin(num='13',name='Y',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='Z',func=Pin.OUTPUT,do_erc=True),
            Pin(num='16',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='B',do_erc=True),
            Pin(num='18',name='A',do_erc=True),
            Pin(num='19',name='Visoin',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ADuM1410',dest=TEMPLATE,tool=SKIDL,keywords='Digital Isolator',description='Quad Channel Digital Isolator,10Mbps,SO-16',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x10.3mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VIA',do_erc=True),
            Pin(num='4',name='VIB',do_erc=True),
            Pin(num='5',name='VIC',do_erc=True),
            Pin(num='6',name='VID',do_erc=True),
            Pin(num='7',name='DISABLE',do_erc=True),
            Pin(num='8',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='CTRL2',do_erc=True),
            Pin(num='11',name='VOD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='VOC',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='VOB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='VOA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD2',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ADuM1411',dest=TEMPLATE,tool=SKIDL,keywords='Digital Isolator',description='Quad Channel Digital Isolator,10Mbps,SO-16',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x10.3mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VIA',do_erc=True),
            Pin(num='4',name='VIB',do_erc=True),
            Pin(num='5',name='VIC',do_erc=True),
            Pin(num='6',name='VOD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='CTRL1',do_erc=True),
            Pin(num='8',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='CTRL2',do_erc=True),
            Pin(num='11',name='VID',do_erc=True),
            Pin(num='12',name='VOC',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='VOB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='VOA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD2',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ADuM1412',dest=TEMPLATE,tool=SKIDL,keywords='Digital Isolator',description='Quad Channel Digital Isolator,10Mbps,SO-16',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x10.3mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VIA',do_erc=True),
            Pin(num='4',name='VIB',do_erc=True),
            Pin(num='5',name='VOC',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='VOD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='CTRL1',do_erc=True),
            Pin(num='8',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='CTRL2',do_erc=True),
            Pin(num='11',name='VID',do_erc=True),
            Pin(num='12',name='VIC',do_erc=True),
            Pin(num='13',name='VOB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='14',name='VOA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='VDD2',func=Pin.PWRIN,do_erc=True)]),
        Part(name='ADuM6000',dest=TEMPLATE,tool=SKIDL,keywords='Isolated DC-to-DC Converter 5kV',description='Isolated 5 kV DC-to-DC Converter',ref_prefix='U',num_units=1,fplist=['SO-16-W*'],do_erc=True,pins=[
            Pin(num='1',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='4',name='RC_IN',do_erc=True),
            Pin(num='5',name='RC_OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='RC_SEL',do_erc=True),
            Pin(num='7',name='VDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GND_ISO',func=Pin.PWROUT,do_erc=True),
            Pin(num='10',name='V_ISO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='12',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='13',name='V_SEL',do_erc=True),
            Pin(num='14',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='15',name='GND_ISO',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='V_ISO',func=Pin.PWROUT,do_erc=True)]),
        Part(name='ADuM7643',dest=TEMPLATE,tool=SKIDL,keywords='6Ch Quad Digital Isolator 25Mbps',description='Low Power Six-Channel 3/3 Digital Isolator, 25Mbps 6ns, Fail-Safe High, QSOP20',ref_prefix='U',num_units=1,fplist=['QSOP*'],do_erc=True,pins=[
            Pin(num='1',name='VDD1A',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='INA',do_erc=True),
            Pin(num='4',name='INB',do_erc=True),
            Pin(num='5',name='OUTC',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='OUTD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='VDD1B',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='OUTE',func=Pin.OUTPUT,do_erc=True),
            Pin(num='9',name='INF',do_erc=True),
            Pin(num='10',name='GND1',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='VDD2A',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='GND2',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='OUTF',func=Pin.OUTPUT,do_erc=True),
            Pin(num='13',name='INE',do_erc=True),
            Pin(num='14',name='VDD2B',do_erc=True),
            Pin(num='15',name='IND',do_erc=True),
            Pin(num='16',name='INC',do_erc=True),
            Pin(num='17',name='OUTB',func=Pin.OUTPUT,do_erc=True),
            Pin(num='18',name='OUTA',func=Pin.OUTPUT,do_erc=True),
            Pin(num='19',name='GND2',func=Pin.PWRIN,do_erc=True)])])


import pytest
from skidl import *
from .setup_teardown import *

def test_string_indices_1():
    vreg1 = Part('xess.lib',
                 '1117',
                 footprint='null')
    gnd = Net('GND')
    vin = Net('Vin')
    vreg1['GND, IN, OUT'] += gnd, vin, vreg1['HS']
    assert vreg1.is_connected() == True
    assert len(gnd) == 1
    assert len(vin) == 1
    assert len(vreg1['IN'].net) == 1
    assert len(vreg1['HS'].net) == 2
    assert len(vreg1['OUT'].net) == 2
    assert vreg1['OUT'].net.is_attached(vreg1['HS'].net)



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

microchip_pic32mcu = SchLib(tool=SKIDL).add_parts(*[
        Part(name='PIC32MX1XXFXXXD-I/PT',dest=TEMPLATE,tool=SKIDL,keywords='Microchip PIC32 Microcontroller',description='32-bit Microcontrollers (up to 256 KB Flash and 64 KB SRAM) with Audio and Graphics Interfaces, USB, and Advanced Analog',ref_prefix='U',num_units=1,fplist=['TQFP*10x10mm*Pitch0.8mm*'],do_erc=True,aliases=['PIC32MX110F016D-I/PT', 'PIC32MX120F032D-I/PT', 'PIC32MX130F064D-I/PT', 'PIC32MX150F128D-I/PT', 'PIC32MX170F256D-I/PT'],pins=[
            Pin(num='1',name='RB9/PMD3/CTED4/SDA1/RPB9',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='RPC6/PMA1/RC6',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='RPC7/PMA0/RC7',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='RPC8/PMA5/RC8',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='RPC9/CTED7/PMA6/RC9',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='VCAP',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='RB10/PMD2/CTED11/RPB10/PGED2',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='RB11/PMD1/RPB11/PGEC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='RB12/PMD0/AN12',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='VREF-/CVREF-/AN1/RPA1/CTED2/RA1',do_erc=True),
            Pin(num='30',name='OSC1/CLKI/RPA2/RA2',do_erc=True),
            Pin(num='40',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='RB13/PMRD/CTPLS/RPB13/AN11',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='RB0/RPB0/C3IND/C2INB/C1IND/AN2/PGED1',do_erc=True),
            Pin(num='31',name='OSC2/CLKO/RPA3/RA3',do_erc=True),
            Pin(num='41',name='RB5/PMD7/RPB5/PGED3',do_erc=True),
            Pin(num='12',name='PGED/TMS/PMA10/RA10',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='RB1/CTED12/RPB1/C2INA/C1INC/AN3/PGEC1',do_erc=True),
            Pin(num='32',name='TDO/RPA8/PMA8/RA8',do_erc=True),
            Pin(num='42',name='RB6/PMD6/RPB6/PGEC3',do_erc=True),
            Pin(num='13',name='PGEC/TCK/CTED8/PMA7/RA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='RB2/CTED13/SDA2/RPB2/C2IND/C1INB/AN4',do_erc=True),
            Pin(num='33',name='RB4/RPB4/SOSCI',do_erc=True),
            Pin(num='43',name='RB7/INT0/PMD5/CTED3/RPB7',do_erc=True),
            Pin(num='14',name='RB14/PMWR/CTED5/SCK1/RPB14/C3INB/AN10/CVREFOUT',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='RB3/SCL2/RPB3/RTCC/C2INC/C1INA/AN5',do_erc=True),
            Pin(num='34',name='SOSCO/RPA4/T1CK/CTED9/RA4',do_erc=True),
            Pin(num='44',name='RB8/PMD4/CTED10/SCL1/RPB8',do_erc=True),
            Pin(num='15',name='RB15/PMCS1/CTED6/SCK2/RPB15/C3INA/AN9',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='AN6/RPC0/RC0',do_erc=True),
            Pin(num='35',name='TDI/RPA9/PMA9/RA9',do_erc=True),
            Pin(num='16',name='AVSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='AN7/RPC1/RC1',do_erc=True),
            Pin(num='36',name='RPC3/RC3',do_erc=True),
            Pin(num='17',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='AN8/RPC2/PMA2/RC2',do_erc=True),
            Pin(num='37',name='RPC4/PMA4/RC4',do_erc=True),
            Pin(num='18',name='MCLR',do_erc=True),
            Pin(num='28',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='38',name='RPC5/PMA3/RC5',do_erc=True),
            Pin(num='19',name='VREF+/CVREF+/AN0/C3INC/RPA0/CTED1/RA0',do_erc=True),
            Pin(num='29',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='39',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC32MX2XXFXXXD-I/PT',dest=TEMPLATE,tool=SKIDL,keywords='Microchip PIC32 Microcontroller',description='32-bit Microcontrollers (up to 256 KB Flash and 64 KB SRAM) with Audio and Graphics Interfaces, USB, and Advanced Analog',ref_prefix='U',num_units=1,fplist=['TQFP*10x10mm*Pitch0.8mm*'],do_erc=True,aliases=['PIC32MX210F016D-I/PT', 'PIC32MX220F032D-I/PT', 'PIC32MX230F064D-I/PT', 'PIC32MX250F128D-I/PT', 'PIC32MX270F256D-I/PT'],pins=[
            Pin(num='1',name='RB9/PMD3/CTED4/SDA1/RPB9',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='RPC6/PMA1/RC6',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='RPC7/PMA0/RC7',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='RPC8/PMA5/RC8',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='RPC9/CTED7/PMA6/RC9',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='VCAP',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='RB10/CTED11/D+/RPB10/PGED2',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='RB11/D-/RPB11/PGEC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='VUSB3V3',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='PGEC3/VREF-/CVREF-/AN1/RPA1/CTED2/PMD6/RA1',do_erc=True),
            Pin(num='30',name='OSC1/CLKI/RPA2/RA2',do_erc=True),
            Pin(num='40',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='RB13/PMRD/CTPLS/RPB13/AN11',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='RB0/PMD0/RPB0/C3IND/C2INB/C1IND/AN2/PGED1',do_erc=True),
            Pin(num='31',name='OSC2/CLKO/RPA3/RA3',do_erc=True),
            Pin(num='41',name='RB5/USBID/RPB5',do_erc=True),
            Pin(num='12',name='PGED/TMS/PMA10/RA10',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='RB1/PMD1/CTED12/RPB1/C2INA/C1INC/AN3/PGEC1',do_erc=True),
            Pin(num='32',name='TDO/RPA8/PMA8/RA8',do_erc=True),
            Pin(num='42',name='VBUS',do_erc=True),
            Pin(num='13',name='PGEC/TCK/CTED8/PMA7/RA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='RB2/PMD2/CTED13/SDA2/RPB2/C2IND/C1INB/AN4',do_erc=True),
            Pin(num='33',name='RB4/RPB4/SOSCI',do_erc=True),
            Pin(num='43',name='RB7/INT0/PMD5/CTED3/RPB7',do_erc=True),
            Pin(num='14',name='RB14/CTED5/SCK1/VBUSON/RPB14/C3INB/AN10/CVREFOUT',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='RB3/PMWR/SCL2/RPB3/RTCC/C2INC/C1INA/AN5',do_erc=True),
            Pin(num='34',name='SOSCO/RPA4/T1CK/CTED9/RA4',do_erc=True),
            Pin(num='44',name='RB8/PMD4/CTED10/SCL1/RPB8',do_erc=True),
            Pin(num='15',name='RB15/PMCS1/CTED6/SCK2/RPB15/C3INA/AN9',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='AN6/RPC0/RC0',do_erc=True),
            Pin(num='35',name='TDI/RPA9/PMA9/RA9',do_erc=True),
            Pin(num='16',name='AVSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='AN7/RPC1/RC1',do_erc=True),
            Pin(num='36',name='AN12/RPC3/RC3',do_erc=True),
            Pin(num='17',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='AN8/RPC2/PMA2/RC2',do_erc=True),
            Pin(num='37',name='RPC4/PMA4/RC4',do_erc=True),
            Pin(num='18',name='MCLR',do_erc=True),
            Pin(num='28',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='38',name='RPC5/PMA3/RC5',do_erc=True),
            Pin(num='19',name='PGED3/VREF+/CVREF+/AN0/C3INC/RPA0/CTED1/PMD7/RA0',do_erc=True),
            Pin(num='29',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='39',name='VSS',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PIC32MX575F-TQFP64',dest=TEMPLATE,tool=SKIDL,keywords='PIC 32-bit Flash Microcontroller USB CAN',description='PIC32MX575F, 256/512KB + 12KB Flash, 64KB SRAM, USB, CAN, TQFP-64',ref_prefix='U',num_units=1,fplist=['TQFP*'],do_erc=True,pins=[
            Pin(num='1',name='PMD5/RE5',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='PMD6/RE6',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='PMD7/RE7',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='SCK2A/U2BTX/U2ARTS/PMA5/CN8/RG6',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='SDA2A/SDI2A/U2ARX/PMA4/CN9/RG7',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='SCL2A/SDO2A/U2ATX/PMA3/CN10/RG8',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='~MCLR~',do_erc=True),
            Pin(num='8',name='SS2A/U2BRX/U2ACTS/PMA2/CN11/RG9',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='AVSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='30',name='RB15/AN15/OCFB/PMALL/PMA0/CN12',func=Pin.BIDIR,do_erc=True),
            Pin(num='40',name='RC15/OSC2/CLK0',func=Pin.BIDIR,do_erc=True),
            Pin(num='50',name='SDA1A/SDI1A/U1ARX/OC3/RD2',func=Pin.BIDIR,do_erc=True),
            Pin(num='60',name='PMD0/RE0',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='RB5/AN5/C1IN+/Vbuson/CN7',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='RB8/AN8/C1OUT/SS3A/U3BRX/U3ACTS',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='AC1TX/SDA3A/SDI3A/U3ARX/PMA9/CN17/RF4',func=Pin.BIDIR,do_erc=True),
            Pin(num='41',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='51',name='SCL1A/SDO1A/U1ATX/OC4/RD3',func=Pin.BIDIR,do_erc=True),
            Pin(num='61',name='PMD1/RE1',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='RB4/AN4/C1IN-/CN6',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='RB9/AN9/C2OUT/PMA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='AC1RX/SCL3A/SDO3A/U3AT/PMA8/CN18/RF5',func=Pin.BIDIR,do_erc=True),
            Pin(num='42',name='RTCC/IC1/INT1/RD8',func=Pin.BIDIR,do_erc=True),
            Pin(num='52',name='OC5/IC5/PMWR/CN13/RD4',func=Pin.BIDIR,do_erc=True),
            Pin(num='62',name='PMD2/RE2',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='RB3/AN3/C2IN+/CN5',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='RB10/AN10/CVrefout/TSM/PMA13',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='USBID/RF3',func=Pin.BIDIR,do_erc=True),
            Pin(num='43',name='SS1A/U1BRX/U1ACTS/SDA1/IC2/INT2/RD9',func=Pin.BIDIR,do_erc=True),
            Pin(num='53',name='PMRD/CN14/RD5',func=Pin.BIDIR,do_erc=True),
            Pin(num='63',name='PMD3/RE3',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='RB2/AN2/C2IN-/CN4',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='RB11/AN11/PMA12/TDO',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='VBUS',do_erc=True),
            Pin(num='44',name='SCL1/IC3/PMCS2/PMA15/INT3/RD10',func=Pin.BIDIR,do_erc=True),
            Pin(num='54',name='CN15/RD6',func=Pin.BIDIR,do_erc=True),
            Pin(num='64',name='PMD4/RE4',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='RB1/AN1/Vref-/CVref-/PGEC1/CN3',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='35',name='VUSB',func=Pin.PWRIN,do_erc=True),
            Pin(num='45',name='IC4/PMCS1/PMA14/INT4/RD11',func=Pin.BIDIR,do_erc=True),
            Pin(num='55',name='CN16/RD7',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='RB0/AN0/Vref+/CVref+/PGED1/PMA6/CN2',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='36',name='D-/RG3',func=Pin.BIDIR,do_erc=True),
            Pin(num='46',name='OC1/INT0/RD0',func=Pin.BIDIR,do_erc=True),
            Pin(num='56',name='VCAP/VDDCORE',func=Pin.PWROUT,do_erc=True),
            Pin(num='17',name='RB6/AN6/OCFA/PGEC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='RB12/AN12/PMA11/TCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='37',name='D+/RG2',func=Pin.BIDIR,do_erc=True),
            Pin(num='47',name='RC13/SOSCI/CN1',func=Pin.BIDIR,do_erc=True),
            Pin(num='57',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='RB7/AN7/PGED2',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='RB13/AN13/PMA10/TDI',func=Pin.BIDIR,do_erc=True),
            Pin(num='38',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='48',name='RC14/SOSCO/T1CK/CN0',func=Pin.BIDIR,do_erc=True),
            Pin(num='58',name='C1RX/RF0',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='29',name='RB14/AN14/SCK3A/U3BTX/U3ARTS/PMALH/PMA1',func=Pin.BIDIR,do_erc=True),
            Pin(num='39',name='RC12/OSC1/CLKI',func=Pin.BIDIR,do_erc=True),
            Pin(num='49',name='SCK1A/U1BTX/U1ARTS/OC2/RD1',func=Pin.BIDIR,do_erc=True),
            Pin(num='59',name='C1TX/RF1',func=Pin.BIDIR,do_erc=True)])])


import pytest
from skidl import *
from .setup_teardown import *

def test_NC_1():

    @subcircuit
    def circ_nc():
        res = Part(tool=SKIDL, name='res', ref_prefix='R', dest=TEMPLATE, pins=[Pin(num=1,func=Pin.PASSIVE), Pin(num=2, func=Pin.PASSIVE)])
        r1 = res()
        r1[1] += NC

    default_circuit.name = 'DEFAULT'
    circuit1 = Circuit(name='CIRCUIT1')
    circ_nc()
    circ_nc(circuit=circuit1)



# -*- coding: utf-8 -*-

# MIT license
#
# Copyright (C) 2016 by XESS Corp.
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
# THE SOFTWARE.

"""
SKiDL: A Python-Based Schematic Design Language

This module extends Python with the ability to design electronic
circuits. It provides classes for working with 

**1)** electronic parts (``Part``),
**2)** collections of part terminals (``Pin``) connected via wires (``Net``), and
**3)** groups of related nets (``Bus``).

Using these classes, you can
concisely describe the interconnection of components using a linear
and/or hierarchical structure. It also provides the capability to
check the resulting circuitry for the violation of electrical rules.
The output of a SKiDL-enabled Python script is a netlist that can be
imported into a PCB layout tool.
"""

from __future__ import print_function
from __future__ import unicode_literals
from __future__ import division
from __future__ import absolute_import

from builtins import open
from future import standard_library
standard_library.install_aliases()

from .py_2_3 import *  # pylint: disable=wildcard-import
from .defines import *
from .utilities import *  # pylint: disable=wildcard-import
from . import tools  # Import EDA tool-specific stuff.
from .SchLib import *
from .Pin import *
from .Alias import *
from .Part import *
from .Net import *
from .Bus import *
from .NetPinList import *
from .Circuit import *
from .part_query import *

###############################################################################
# Globals that are used by everything else.
###############################################################################

# Supported ECAD tools.
DEFAULT_TOOL = INITIAL_DEFAULT_TOOL = KICAD


def set_default_tool(tool):
    """Set the ECAD tool that will be used by default."""
    global DEFAULT_TOOL
    DEFAULT_TOOL = tool


def get_default_tool():
    return DEFAULT_TOOL


# These are the paths to search for part libraries of the ECAD tools.
# Start off with a path that allows absolute file names, and then searches
# within the current directory.
lib_search_paths = {KICAD: ['', '.'], SKIDL: ['', '.'], SPICE: ['', '.'],}

# Add the location of the default KiCad schematic part libs to the search path.
try:
    lib_search_paths[KICAD].append(
        os.path.join(os.environ['KISYSMOD'], '..', 'library'))
except KeyError:
    logger.warning(
        "KISYSMOD environment variable is missing, so default KiCad libraries won't be searched."
    )

# Add the location of the default SKiDL part libraries.
default_skidl_libs = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), 'libs')
lib_search_paths[SKIDL].append(default_skidl_libs)

# Make the various EDA tool library suffixes globally available.
lib_suffixes = tools.lib_suffixes

# Definitions for backup library of circuit parts.
BACKUP_LIB_NAME = get_script_name() + '_lib'
BACKUP_LIB_FILE_NAME = BACKUP_LIB_NAME + lib_suffixes[SKIDL]

# Boolean controls whether backup lib will be searched for missing parts.
QUERY_BACKUP_LIB = INITIAL_QUERY_BACKUP_LIB = True


def set_query_backup_lib(val):
    """Set the boolean that controls searching for the backup library."""
    global QUERY_BACKUP_LIB
    QUERY_BACKUP_LIB = val


def get_query_backup_lib():
    return QUERY_BACKUP_LIB


# Backup lib for storing parts in a Circuit.
backup_lib = None


def set_backup_lib(lib):
    """Set the backup library."""
    global backup_lib
    backup_lib = lib


def get_backup_lib():
    return backup_lib


@norecurse
def load_backup_lib():
    """Load a backup library that stores the parts used in the circuit."""

    global backup_lib

    # Don't keep reloading the backup library once it's loaded.
    if not backup_lib:
        try:
            # The backup library is a SKiDL lib stored as a Python module.
            exec(open(BACKUP_LIB_FILE_NAME).read())
            # Copy the backup library in the local storage to the global storage.
            backup_lib = locals()[BACKUP_LIB_NAME]

        except (FileNotFoundError, ImportError, NameError, IOError):
            pass

    return backup_lib


# Create the default Circuit object that will be used unless another is explicitly created.
builtins.default_circuit = Circuit()
# NOCONNECT net for attaching pins that are intentionally left open.
builtins.NC = default_circuit.NC  # pylint: disable=undefined-variable

# Create calls to functions on whichever Circuit object is the current default.
ERC = default_circuit.ERC  # pylint: disable=undefined-variable
generate_netlist = default_circuit.generate_netlist  # pylint: disable=undefined-variable
generate_xml = default_circuit.generate_xml  # pylint: disable=undefined-variable
generate_graph = default_circuit.generate_graph  # pylint: disable=undefined-variable
reset = default_circuit.reset  # pylint: disable=undefined-variable
backup_parts = default_circuit.backup_parts  # pylint: disable=undefined-variable

# Define a tag for nets that convey power (e.g., VCC or GND).
POWER = Pin.POWER_DRIVE



from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

modules = SchLib(tool=SKIDL).add_parts(*[
        Part(name='Arduino_Nano_v3.x',dest=TEMPLATE,tool=SKIDL,keywords='Arduino nano microcontroller module USB',description='Arduino Nano v3.x',ref_prefix='A',num_units=1,fplist=['Arduino*Nano*'],do_erc=True,aliases=['Arduino_Nano_v2.x'],pins=[
            Pin(num='1',name='D1/TX',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='D0/RX',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='RESET',do_erc=True),
            Pin(num='4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='D2',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='D3',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='D4',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='D5',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='D6',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='D7',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='A1',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='VIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='D8',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='A2',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='D9',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='A3',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='D10',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='A4',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='D11',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='A5',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='D12',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='A6',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='D13',func=Pin.BIDIR,do_erc=True),
            Pin(num='26',name='A7',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='3V3',func=Pin.PWROUT,do_erc=True),
            Pin(num='27',name='+5V',func=Pin.PWROUT,do_erc=True),
            Pin(num='18',name='AREF',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='RESET',do_erc=True),
            Pin(num='19',name='A0',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='NUCLEO64-FR411RE',dest=TEMPLATE,tool=SKIDL,keywords='STM32 Nucleo ST',description='Nucleo 64 Development Board with STM32F411RET6 MCU, 128kB RAM, 512KB FLASH',ref_prefix='U',num_units=1,fplist=['ST?Morpho?Connector?64?With?STLink*'],do_erc=True,pins=[
            Pin(num='1',name='PC10',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='PC11',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='PC12',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='PD2',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='E5V',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='~BOOT0',do_erc=True),
            Pin(num='8',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='10',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='20',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='30',name='PA1',func=Pin.BIDIR,do_erc=True),
            Pin(num='40',name='PC8',func=Pin.BIDIR,do_erc=True),
            Pin(num='50',name='PA12',func=Pin.BIDIR,do_erc=True),
            Pin(num='60',name='PB2',func=Pin.BIDIR,do_erc=True),
            Pin(num='70',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='80',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='21',name='PB7',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='PH1',func=Pin.BIDIR,do_erc=True),
            Pin(num='41',name='PB8',func=Pin.BIDIR,do_erc=True),
            Pin(num='51',name='PA6',func=Pin.BIDIR,do_erc=True),
            Pin(num='61',name='PA8',func=Pin.BIDIR,do_erc=True),
            Pin(num='71',name='PA10',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='IOREF',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='32',name='PA4',func=Pin.BIDIR,do_erc=True),
            Pin(num='42',name='PC6',func=Pin.BIDIR,do_erc=True),
            Pin(num='52',name='PA11',func=Pin.BIDIR,do_erc=True),
            Pin(num='62',name='PB1',func=Pin.BIDIR,do_erc=True),
            Pin(num='72',name='PC4',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='STLINK_TMS/PA13',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='PC13',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='VBAT',func=Pin.PWRIN,do_erc=True),
            Pin(num='43',name='PB9',func=Pin.BIDIR,do_erc=True),
            Pin(num='53',name='PA7',func=Pin.BIDIR,do_erc=True),
            Pin(num='63',name='PB10',func=Pin.BIDIR,do_erc=True),
            Pin(num='73',name='STLINK_UART_TX/PA2',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='~RESET',do_erc=True),
            Pin(num='24',name='VIN',func=Pin.PWRIN,do_erc=True),
            Pin(num='34',name='PB0',func=Pin.BIDIR,do_erc=True),
            Pin(num='44',name='PC5',func=Pin.BIDIR,do_erc=True),
            Pin(num='54',name='PB12',func=Pin.BIDIR,do_erc=True),
            Pin(num='64',name='PB15',func=Pin.BIDIR,do_erc=True),
            Pin(num='74',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='15',name='STLINK_TCK/PA14',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='PC14',func=Pin.BIDIR,do_erc=True),
            Pin(num='35',name='PC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='45',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='55',name='PB6',func=Pin.BIDIR,do_erc=True),
            Pin(num='65',name='PB4',func=Pin.BIDIR,do_erc=True),
            Pin(num='75',name='STLINK_UART_RX/PA3',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='+3V3',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='36',name='PC1',func=Pin.BIDIR,do_erc=True),
            Pin(num='46',name='U5V',func=Pin.PWRIN,do_erc=True),
            Pin(num='56',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='66',name='PB14',func=Pin.BIDIR,do_erc=True),
            Pin(num='76',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='17',name='PA15',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='PC15',func=Pin.BIDIR,do_erc=True),
            Pin(num='37',name='PC3',func=Pin.BIDIR,do_erc=True),
            Pin(num='47',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='57',name='PC7',func=Pin.BIDIR,do_erc=True),
            Pin(num='67',name='PB5',func=Pin.BIDIR,do_erc=True),
            Pin(num='77',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='+5V',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='PA0',func=Pin.BIDIR,do_erc=True),
            Pin(num='38',name='PC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='48',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='58',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='68',name='PB13',func=Pin.BIDIR,do_erc=True),
            Pin(num='78',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='29',name='PH0',func=Pin.BIDIR,do_erc=True),
            Pin(num='39',name='PC9',func=Pin.BIDIR,do_erc=True),
            Pin(num='49',name='PA5',func=Pin.BIDIR,do_erc=True),
            Pin(num='59',name='PA9',func=Pin.BIDIR,do_erc=True),
            Pin(num='69',name='STLINK_SWO/PB3',func=Pin.BIDIR,do_erc=True),
            Pin(num='79',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='Pololu_Breakout_A4988',dest=TEMPLATE,tool=SKIDL,keywords='Pololu Breakout Board Stepper Driver A4988',description='Pololu Breakout Board, Stepper Driver A4988',ref_prefix='A',num_units=1,fplist=['Pololu*Breakout*15.2x20.3mm*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='1B',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='1A',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='2A',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='2B',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VMOT',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='~ENABLE',do_erc=True),
            Pin(num='10',name='MS1',do_erc=True),
            Pin(num='11',name='MS2',do_erc=True),
            Pin(num='12',name='MS3',do_erc=True),
            Pin(num='13',name='~RESET',do_erc=True),
            Pin(num='14',name='~SLEEP',do_erc=True),
            Pin(num='15',name='STEP',do_erc=True),
            Pin(num='16',name='DIR',do_erc=True)]),
        Part(name='Pololu_Breakout_DRV8825',dest=TEMPLATE,tool=SKIDL,keywords='Pololu Breakout Board Stepper Driver DRV8825',description='Pololu Breakout Board, Stepper Driver DRV8825',ref_prefix='A',num_units=1,fplist=['Pololu*Breakout*15.2x20.3mm*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='~FLT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='3',name='B2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='B1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='A1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='A2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VMOT',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='~EN',do_erc=True),
            Pin(num='10',name='M0',do_erc=True),
            Pin(num='11',name='M1',do_erc=True),
            Pin(num='12',name='M2',do_erc=True),
            Pin(num='13',name='~RST',do_erc=True),
            Pin(num='14',name='~SLP',do_erc=True),
            Pin(num='15',name='STEP',do_erc=True),
            Pin(num='16',name='DIR',do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

NXP = SchLib(tool=SKIDL).add_parts(*[
        Part(name='PCA9536D',dest=TEMPLATE,tool=SKIDL,keywords='i2c io port',description='4-bit I2C-bus and SMBus IO port, SOIC8 package',ref_prefix='U',num_units=1,fplist=['SOIC*3.9x4.9mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='IO0',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='IO1',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='IO2',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='IO3',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PCA9536DP',dest=TEMPLATE,tool=SKIDL,keywords='i2c io port',description='4-bit I2C-bus and SMBus IO port, TSSOP8 package',ref_prefix='U',num_units=1,fplist=['TSSOP*3x3mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='1',name='IO0',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='IO1',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='IO2',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='IO3',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='VDD',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PCA9544AD',dest=TEMPLATE,tool=SKIDL,keywords='i2c multiplexer',description='4-channel I2C-bus multiplexer with interrupt logic, SOIC20 package',ref_prefix='U',num_units=1,fplist=['SOIC*7.5x12.8mm*Pitch1.27mm*'],do_erc=True,pins=[
            Pin(num='1',name='A0',do_erc=True),
            Pin(num='2',name='A1',do_erc=True),
            Pin(num='3',name='A2',do_erc=True),
            Pin(num='4',name='~INT0',do_erc=True),
            Pin(num='5',name='SD0',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='SC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='~INT1',do_erc=True),
            Pin(num='8',name='SD1',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='SC1',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='~INT2',do_erc=True),
            Pin(num='12',name='SD2',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='SC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='~INT3',do_erc=True),
            Pin(num='15',name='SD3',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='SC3',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='~INT',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='18',name='SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='SDA',func=Pin.BIDIR,do_erc=True)]),
        Part(name='PCA9544APW',dest=TEMPLATE,tool=SKIDL,keywords='i2c multiplexer',description='4-channel I2C-bus multiplexer with interrupt logic, TSSOP20 package',ref_prefix='U',num_units=1,fplist=['TSSOP*4.4x6.5mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='1',name='A0',do_erc=True),
            Pin(num='2',name='A1',do_erc=True),
            Pin(num='3',name='A2',do_erc=True),
            Pin(num='4',name='~INT0',do_erc=True),
            Pin(num='5',name='SD0',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='SC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='~INT1',do_erc=True),
            Pin(num='8',name='SD1',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='SC1',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='~INT2',do_erc=True),
            Pin(num='12',name='SD2',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='SC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='~INT3',do_erc=True),
            Pin(num='15',name='SD3',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='SC3',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='~INT',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='18',name='SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='SDA',func=Pin.BIDIR,do_erc=True)]),
        Part(name='PCA9685BS',dest=TEMPLATE,tool=SKIDL,keywords='PWM LED driver I2C QFN',description='16-channel 12-bit PWM Fm+ I2C-bus LED controller RGBA QFN',ref_prefix='U',num_units=1,fplist=['QFN*6x6mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='1',name='A0',do_erc=True),
            Pin(num='2',name='A1',do_erc=True),
            Pin(num='3',name='A2',do_erc=True),
            Pin(num='4',name='A3',do_erc=True),
            Pin(num='5',name='A4',do_erc=True),
            Pin(num='6',name='OUT0',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='7',name='OUT1',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='8',name='OUT2',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='9',name='OUT3',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='10',name='OUT4',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='20',name='OUT13',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='11',name='OUT5',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='21',name='OUT14',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='12',name='OUT6',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='22',name='OUT15',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='13',name='OUT7',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='23',name='~OE~',do_erc=True),
            Pin(num='14',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='A5',do_erc=True),
            Pin(num='15',name='OUT8',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='25',name='EXTCLK',do_erc=True),
            Pin(num='16',name='OUT9',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='26',name='SCL',do_erc=True),
            Pin(num='17',name='OUT10',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='27',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='OUT11',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='28',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='OUT12',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='PCA9685PW',dest=TEMPLATE,tool=SKIDL,keywords='PWM LED driver I2C TSSOP',description='16-channel 12-bit PWM Fm+ I2C-bus LED controller RGBA TSSOP',ref_prefix='U',num_units=1,fplist=['TSSOP*4.4x9.7mm*Pitch0.65mm*'],do_erc=True,pins=[
            Pin(num='1',name='A0',do_erc=True),
            Pin(num='2',name='A1',do_erc=True),
            Pin(num='3',name='A2',do_erc=True),
            Pin(num='4',name='A3',do_erc=True),
            Pin(num='5',name='A4',do_erc=True),
            Pin(num='6',name='OUT0',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='7',name='OUT1',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='8',name='OUT2',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='9',name='OUT3',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='10',name='OUT4',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='20',name='OUT13',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='11',name='OUT5',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='21',name='OUT14',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='12',name='OUT6',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='22',name='OUT15',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='13',name='OUT7',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='23',name='~OE~',do_erc=True),
            Pin(num='14',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='A5',do_erc=True),
            Pin(num='15',name='OUT8',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='25',name='EXTCLK',do_erc=True),
            Pin(num='16',name='OUT9',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='26',name='SCL',do_erc=True),
            Pin(num='17',name='OUT10',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='27',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='OUT11',func=Pin.OPENCOLL,do_erc=True),
            Pin(num='28',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='OUT12',func=Pin.OPENCOLL,do_erc=True)])])


from skidl import Pin, Part, SchLib, SKIDL, TEMPLATE

SKIDL_lib_version = '0.0.1'

rfcom = SchLib(tool=SKIDL).add_parts(*[
        Part(name='BL652',dest=TEMPLATE,tool=SKIDL,keywords='Bluetooth Nordic nRF52',description='Bluetooth module',ref_prefix='U',num_units=1,fplist=['Laird*BL652*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='SIO_24',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='SIO_23',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='SIO_22',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='SWDIO',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='SWDCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='SIO_21',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='SIO_20',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='SIO_18',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='SIO_16',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='SIO_05/AIN3',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='SIO_17',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='SIO_14',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='SIO_04/AIN2',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='SIO_19',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='SIO_12',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='SIO_03/AIN1',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='SIO_31/AIN7',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='SIO_11',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='SIO_02/AIN0',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='SIO_30/AIN6',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='SIO_10/NFC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='SIO_01',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='SIO_29/AIN5',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='SIO_09/NFC1',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='SIO_00',func=Pin.BIDIR,do_erc=True),
            Pin(num='35',name='SIO_28/AIN4',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='36',name='SIO_27',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='SIO_08',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='37',name='SIO_26',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='SIO_07',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='SIO_13',func=Pin.BIDIR,do_erc=True),
            Pin(num='38',name='SIO_25',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='SIO_06',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='SIO_15',func=Pin.BIDIR,do_erc=True),
            Pin(num='39',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='BTM112',dest=TEMPLATE,tool=SKIDL,keywords='Bluetooth BT SPP Module',description='Bluetooth SPP Module, UART, Class 2',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='PIO8',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='PIO9',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='PIO10',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='AIO0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='AIO1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='6',name='RESET',do_erc=True),
            Pin(num='7',name='SPI_MISO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='~SPI_CSB~',do_erc=True),
            Pin(num='9',name='SPI_CLK',do_erc=True),
            Pin(num='10',name='SPI_MOSI',do_erc=True),
            Pin(num='20',name='PCM_IN',do_erc=True),
            Pin(num='30',name='PIO1',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='~UART_CTS',do_erc=True),
            Pin(num='21',name='PCM_CLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='PIO0',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='UART_TX',func=Pin.OUTPUT,do_erc=True),
            Pin(num='22',name='USB_D+',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='~UART_RTS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='23',name='USB_D-',func=Pin.BIDIR,do_erc=True),
            Pin(num='33',name='RF',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='UART_RX',do_erc=True),
            Pin(num='24',name='~LINK~/PIO7',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='PIO11',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='CONN/PIO6',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='PIO5',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='BTN/PIO4',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='PCM_OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='28',name='PIO3',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='PCM_SYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='PIO2',func=Pin.BIDIR,do_erc=True)]),
        Part(name='BTM222',dest=TEMPLATE,tool=SKIDL,keywords='Bluetooth BT SPP Module',description='Bluetooth SPP Module, UART, Class 1',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='PVCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='AIO0/SLEEPCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='AIO1',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='PIO0/RXEN',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='PIO1/TXEN',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='PIO2/USB_PU/CLK_REQ_OUT',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='PIO3/USB_WKUP/CLK_REQ_IN',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='PIO4/USB_ON/BT_PRIOR',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='20',name='USB_D+',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='UART_CTS',do_erc=True),
            Pin(num='11',name='PIO5/USB_DETACH/BT_ACT',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='USB_D-',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='SPI_MOSI',do_erc=True),
            Pin(num='12',name='PIO6/CLK_REQ/WAN_ACT',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='PCM_SYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='~SPI_CSB~',do_erc=True),
            Pin(num='13',name='PIO7',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='PCM_IN',do_erc=True),
            Pin(num='33',name='SPI_CLK',do_erc=True),
            Pin(num='14',name='PIO8',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='PCM_OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='34',name='SPI_MISO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='PIO9',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='PCM_CLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='35',name='PIO11',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='~RESET~',do_erc=True),
            Pin(num='26',name='UART_RX',do_erc=True),
            Pin(num='36',name='PIO10',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='UART_TX',func=Pin.OUTPUT,do_erc=True),
            Pin(num='37',name='RF',func=Pin.PASSIVE,do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='UART_RTS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='38',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='CC1000',dest=TEMPLATE,tool=SKIDL,keywords='Low Power RF Transciever',description='Single Chip Low Power RF Transceiver, TSSOP28',ref_prefix='U',num_units=1,fplist=['TSSOP*'],do_erc=True,pins=[
            Pin(num='1',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='RF_IN',func=Pin.PASSIVE,do_erc=True),
            Pin(num='4',name='RF_OUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='5',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='L1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='L2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='21',name='DVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='CHP_OUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='22',name='DGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='R_BIAS',func=Pin.PASSIVE,do_erc=True),
            Pin(num='23',name='DIO',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='DCLK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='15',name='AVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='PCLK',do_erc=True),
            Pin(num='16',name='AGND',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='PDATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='XOSC_Q2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='27',name='PALE',do_erc=True),
            Pin(num='18',name='XOSC_Q1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='28',name='RSSI/IF',func=Pin.PASSIVE,do_erc=True),
            Pin(num='19',name='AGND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='CC1200',dest=TEMPLATE,tool=SKIDL,keywords='RF Tx Rx',description='Low-Power, High-Performance RF Transceiver',ref_prefix='U',num_units=1,fplist=['QFN-32-1EP_5x5mm_Pitch0.5mm', 'QFN-32-1EP_5x5mm_Pitch0.5mm*'],do_erc=True,pins=[
            Pin(num='1',name='VDD_GUARD',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='~RESET~',do_erc=True),
            Pin(num='3',name='GPIO3',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='GPIO2',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='DVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='6',name='DCPL',func=Pin.PWROUT,do_erc=True),
            Pin(num='7',name='SI',do_erc=True),
            Pin(num='8',name='SCLK',do_erc=True),
            Pin(num='9',name='SO(GPIO1)',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='GPIO0',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='LNA_N',func=Pin.PASSIVE,do_erc=True),
            Pin(num='30',name='XOSC_Q1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='~CS~',do_erc=True),
            Pin(num='21',name='DCPL_VCO',func=Pin.PWROUT,do_erc=True),
            Pin(num='31',name='XOSC_Q2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='12',name='DVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='AVDD_SYNTH1',func=Pin.PWRIN,do_erc=True),
            Pin(num='32',name='EXT_XOSC',do_erc=True),
            Pin(num='13',name='AVDD_IF',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='LPF0',func=Pin.PASSIVE,do_erc=True),
            Pin(num='33',name='GND_EP',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='RBIAS',func=Pin.PASSIVE,do_erc=True),
            Pin(num='24',name='LPF1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='15',name='AVDD_RF',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='AVDD_PFD_CHP',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='DCPL_PFD_CHP',func=Pin.PWROUT,do_erc=True),
            Pin(num='17',name='PA',func=Pin.PASSIVE,do_erc=True),
            Pin(num='27',name='AVDD_SYNTH2',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='TRX_SW',func=Pin.PASSIVE,do_erc=True),
            Pin(num='28',name='AVDD_XOSC',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='LNA_P',func=Pin.PASSIVE,do_erc=True),
            Pin(num='29',name='DCPL_XOSC',func=Pin.PWROUT,do_erc=True)]),
        Part(name='CC2520',dest=TEMPLATE,tool=SKIDL,keywords='2.4GHz rf transceiver ZigBee 802.15.4',description='2.4 GHz ZigBee/IEEE 802.15.4 RF transceiver',ref_prefix='U',num_units=1,fplist=['*QFN*28*5x5mm*Pitch0.5mm*'],do_erc=True,pins=[
            Pin(num='1',name='SO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='2',name='SI',do_erc=True),
            Pin(num='3',name='~CS',do_erc=True),
            Pin(num='4',name='GPIO5',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='GPIO4',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='GPIO3',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='GPIO2',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='DVDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='GPIO1',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='GPIO0',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='AVDD1',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='AVDD5',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='12',name='XOSC_Q2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='22',name='AVDD4',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='XOSC_Q1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='23',name='RBIAS',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='AVDD3',func=Pin.PWRIN,do_erc=True),
            Pin(num='24',name='AVDD_GUARD',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='25',name='~RESET',do_erc=True),
            Pin(num='16',name='AVDD2',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='VREG_EN',do_erc=True),
            Pin(num='17',name='RF_P',func=Pin.PASSIVE,do_erc=True),
            Pin(num='27',name='DCOUPL',func=Pin.PASSIVE,do_erc=True),
            Pin(num='28',name='SCLK',do_erc=True),
            Pin(num='19',name='RF_N',func=Pin.PASSIVE,do_erc=True),
            Pin(num='29',name='AGND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='HF-A11-SMT',dest=TEMPLATE,tool=SKIDL,keywords='WiFi IEEE802.11 b/g/n',description='WiFi IEEE802.11b/g/n with Ethernet Module, UART, GPIO',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='3.3V',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='3.3V',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='UART_TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='UART_RXD',do_erc=True),
            Pin(num='6',name='UART_RTS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='UART_CTS',do_erc=True),
            Pin(num='8',name='TX+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='9',name='TX-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='RX+',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='RX-',func=Pin.PASSIVE,do_erc=True),
            Pin(num='21',name='UART1_RXD',do_erc=True),
            Pin(num='22',name='UART1_TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='23',name='1.8VOUT',func=Pin.PWROUT,do_erc=True),
            Pin(num='14',name='~LINK~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='24',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='~RESET~',do_erc=True),
            Pin(num='25',name='RF',func=Pin.PASSIVE,do_erc=True),
            Pin(num='16',name='~READY~',func=Pin.OUTPUT,do_erc=True),
            Pin(num='26',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='17',name='~RELOAD~',do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='MM002',dest=TEMPLATE,tool=SKIDL,keywords='IOT LoRa SIGFOX',description='NEMEUS Modem dual-mode LoRa/SIGFOX',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='~NRST',do_erc=True),
            Pin(num='3',name='PB9-IO/I2C-SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='PB8-IO/I2C-SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='BOOT',do_erc=True),
            Pin(num='6',name='PB7-IO/UART1-RX',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='PB6-IO/UART1-TX',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='PB4-IO/NJTRST',do_erc=True),
            Pin(num='9',name='PB3-IO/JTDO',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='PA15-IO/JTDI',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='PA5-IO/SPI-SCK',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='PA14-IO/JTCK/SWCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='21',name='PA6-IO/SPI-MISO',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='PA13-IO/JTMS/SWDAT',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='PA4-IO/SPI-NSS',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='PA12-IO/UART1-RTS/USB-DP',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='PA3-IO/ADC/UART2-RX',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='PA11-IO/UART1-CTS/USB-DM',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='PA2-IO/ADC/UART2-TX',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='PA0-IO/ADC/UART2-CTS/WKUP',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='PA1-IO/ADC/UART2-RTS',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='ANT',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='PA7-IO/SPI-MOSI',func=Pin.BIDIR,do_erc=True)]),
        Part(name='NRF24L01',dest=TEMPLATE,tool=SKIDL,keywords='Low Power RF Transciever',description='nRF24L01+, Ultra low power 2.4GHz RF Transceiver, QFN20 4x4mm',ref_prefix='U',num_units=1,fplist=['QFN*4x4*0.5mm*'],do_erc=True,aliases=['nRF24L01P'],pins=[
            Pin(num='1',name='CE',do_erc=True),
            Pin(num='2',name='CSN',do_erc=True),
            Pin(num='3',name='SCK',do_erc=True),
            Pin(num='4',name='MOSI',do_erc=True),
            Pin(num='5',name='MISO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='IRQ',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='8',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='9',name='XC2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='10',name='XC1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='20',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='VDD_PA',func=Pin.PWROUT,do_erc=True),
            Pin(num='12',name='ANT1',func=Pin.PASSIVE,do_erc=True),
            Pin(num='13',name='ANT2',func=Pin.PASSIVE,do_erc=True),
            Pin(num='14',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='IREF',func=Pin.PASSIVE,do_erc=True),
            Pin(num='17',name='VSS',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='DVDD',func=Pin.PWROUT,do_erc=True)]),
        Part(name='NRF24L01_Breakout',dest=TEMPLATE,tool=SKIDL,keywords='Low Power RF Transciever breakout carrier',description='Ultra low power 2.4GHz RF Transceiver, Carrier PCB',ref_prefix='U',num_units=1,fplist=['nRF24L01*Breakout*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='CE',do_erc=True),
            Pin(num='4',name='~CSN',do_erc=True),
            Pin(num='5',name='SCK',do_erc=True),
            Pin(num='6',name='MOSI',do_erc=True),
            Pin(num='7',name='MISO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='IRQ',func=Pin.OUTPUT,do_erc=True)]),
        Part(name='RN42',dest=TEMPLATE,tool=SKIDL,keywords='Bluetooth Module',description='Class 2 Bluetooth Module with on-board antenna',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='SPI_MOSI',do_erc=True),
            Pin(num='3',name='GPIO6',do_erc=True),
            Pin(num='4',name='GPIO7',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='RESET',do_erc=True),
            Pin(num='6',name='SPI_CLK',do_erc=True),
            Pin(num='7',name='PCM_CLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='PCM_SYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='PCM_IN',do_erc=True),
            Pin(num='10',name='PCM_OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='20',name='GPIO3',do_erc=True),
            Pin(num='30',name='AIO0',do_erc=True),
            Pin(num='11',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='GPIO5',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='GPIO8',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='GPIO4',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='GPIO9',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='UART_RX',do_erc=True),
            Pin(num='23',name='SPI_CSB',do_erc=True),
            Pin(num='33',name='GPIO10',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='UART_TX',func=Pin.OUTPUT,do_erc=True),
            Pin(num='24',name='SPI_MISO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='34',name='GPIO11',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='UART_RTS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='35',name='AIO1',do_erc=True),
            Pin(num='16',name='UART_CTS',do_erc=True),
            Pin(num='36',name='SHIELD',do_erc=True),
            Pin(num='17',name='USB_D+',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='USB_D-',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='GPIO2',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='RN42N',dest=TEMPLATE,tool=SKIDL,keywords='Bluetooth Module',description='Class 2 Bluetooth Module without antenna',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='SPI_MOSI',do_erc=True),
            Pin(num='3',name='GPIO6',do_erc=True),
            Pin(num='4',name='GPIO7',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='RESET',do_erc=True),
            Pin(num='6',name='SPI_CLK',do_erc=True),
            Pin(num='7',name='PCM_CLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='PCM_SYNC',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='PCM_IN',do_erc=True),
            Pin(num='10',name='PCM_OUT',func=Pin.OUTPUT,do_erc=True),
            Pin(num='20',name='GPIO3',do_erc=True),
            Pin(num='30',name='AIO0',do_erc=True),
            Pin(num='11',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='GPIO5',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='GPIO8',func=Pin.OUTPUT,do_erc=True),
            Pin(num='12',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='GPIO4',func=Pin.BIDIR,do_erc=True),
            Pin(num='32',name='GPIO9',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='UART_RX',do_erc=True),
            Pin(num='23',name='SPI_CSB',do_erc=True),
            Pin(num='33',name='GPIO10',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='UART_TX',func=Pin.OUTPUT,do_erc=True),
            Pin(num='24',name='SPI_MISO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='34',name='GPIO11',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='UART_RTS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='25',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='35',name='AIO1',do_erc=True),
            Pin(num='16',name='UART_CTS',do_erc=True),
            Pin(num='26',name='RF_ANT',func=Pin.BIDIR,do_erc=True),
            Pin(num='36',name='SHIELD',do_erc=True),
            Pin(num='17',name='USB_D+',func=Pin.BIDIR,do_erc=True),
            Pin(num='27',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='USB_D-',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='19',name='GPIO2',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='SA605D',dest=TEMPLATE,tool=SKIDL,do_erc=True),
        Part(name='SIM900',dest=TEMPLATE,tool=SKIDL,keywords='GSM GPRS Quad-Band SMS FAX',description='GSM Quad-Band Communication Module, GPRS, Audio Engine, AT Command Set',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='PWRKEY',func=Pin.PASSIVE,do_erc=True),
            Pin(num='2',name='PWRKEY_OUT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='3',name='DTR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='4',name='RI',func=Pin.OUTPUT,do_erc=True),
            Pin(num='5',name='DCD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='6',name='DSR',func=Pin.OUTPUT,do_erc=True),
            Pin(num='7',name='CTS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='8',name='RTS',do_erc=True),
            Pin(num='9',name='TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='10',name='RXD',do_erc=True),
            Pin(num='20',name='MIC_N',func=Pin.PASSIVE,do_erc=True),
            Pin(num='30',name='SIM_VDD',func=Pin.PWROUT,do_erc=True),
            Pin(num='40',name='GPIO1/KBR4',func=Pin.BIDIR,do_erc=True),
            Pin(num='50',name='GPIO9/KBC1',func=Pin.BIDIR,do_erc=True),
            Pin(num='60',name='RF_ANT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='11',name='DS_CLK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='21',name='SPK_P',func=Pin.PASSIVE,do_erc=True),
            Pin(num='31',name='SIM_DATA',func=Pin.BIDIR,do_erc=True),
            Pin(num='41',name='GPIO2/KBR3',func=Pin.BIDIR,do_erc=True),
            Pin(num='51',name='GPIO10/KBC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='61',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='12',name='DS_DTA',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='SPK_N',func=Pin.PASSIVE,do_erc=True),
            Pin(num='32',name='SIM_CLK',func=Pin.OUTPUT,do_erc=True),
            Pin(num='42',name='GPIO3/KBR2',func=Pin.BIDIR,do_erc=True),
            Pin(num='52',name='NETLIGHT',func=Pin.PASSIVE,do_erc=True),
            Pin(num='62',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='DS_D/C',func=Pin.OUTPUT,do_erc=True),
            Pin(num='23',name='LINE_R',func=Pin.PASSIVE,do_erc=True),
            Pin(num='33',name='SIM_RST',func=Pin.OUTPUT,do_erc=True),
            Pin(num='43',name='GPIO4/KBR1',func=Pin.BIDIR,do_erc=True),
            Pin(num='53',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='63',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='DS_CS',func=Pin.OUTPUT,do_erc=True),
            Pin(num='24',name='LINE_L',func=Pin.PASSIVE,do_erc=True),
            Pin(num='34',name='SIM_PRESENCE',func=Pin.OUTPUT,do_erc=True),
            Pin(num='44',name='GPIO5/KBR0',func=Pin.BIDIR,do_erc=True),
            Pin(num='54',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='64',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='15',name='VDD_EXT',func=Pin.PWRIN,do_erc=True),
            Pin(num='25',name='ADC',func=Pin.PASSIVE,do_erc=True),
            Pin(num='35',name='PWM1',func=Pin.OUTPUT,do_erc=True),
            Pin(num='45',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='55',name='VBAT',func=Pin.PWRIN,do_erc=True),
            Pin(num='65',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='~RESET~',do_erc=True),
            Pin(num='26',name='VRTC',func=Pin.PASSIVE,do_erc=True),
            Pin(num='36',name='PWM2',func=Pin.OUTPUT,do_erc=True),
            Pin(num='46',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='56',name='VBAT',func=Pin.PWRIN,do_erc=True),
            Pin(num='66',name='STATUS',func=Pin.PASSIVE,do_erc=True),
            Pin(num='17',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='DBG_TXD',func=Pin.OUTPUT,do_erc=True),
            Pin(num='37',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='47',name='GPIO6/KBC4',func=Pin.BIDIR,do_erc=True),
            Pin(num='57',name='VBAT',func=Pin.PWRIN,do_erc=True),
            Pin(num='67',name='GPIO11',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='28',name='DBG_RXD',do_erc=True),
            Pin(num='38',name='SCL',func=Pin.OUTPUT,do_erc=True),
            Pin(num='48',name='GPIO7/KBC3',func=Pin.BIDIR,do_erc=True),
            Pin(num='58',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='68',name='GPIO12',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='MIC_P',func=Pin.PASSIVE,do_erc=True),
            Pin(num='29',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='39',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='49',name='GPIO8/KBC2',func=Pin.BIDIR,do_erc=True),
            Pin(num='59',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='TD1205',dest=TEMPLATE,tool=SKIDL,keywords='IOT SIGFOX GPS',description='High-Performance, Low-Current SIGFOX™ Gateway And GPS Receiver With Integrated Antennas',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='BAT-',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='BAT+',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='~RST',do_erc=True),
            Pin(num='6',name='UART-TX',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='UART-RX',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='DB2-SWDIO',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='DB3-SWCLK',func=Pin.BIDIR,do_erc=True)]),
        Part(name='TD1208',dest=TEMPLATE,tool=SKIDL,keywords='IOT SIGFOX',description='High-Performance, Low-Current SIGFOX™ Gateway',ref_prefix='U',num_units=1,do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='Reserved',func=Pin.UNSPEC,do_erc=True),
            Pin(num='4',name='USR4',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='DB3-SWCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='DB2-SWDIO',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='10',name='USR2',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='ADC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='TIM2',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='22',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='USR3',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='RF_GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='14',name='~RST',do_erc=True),
            Pin(num='24',name='RF',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='DAC0',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='RF_GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='USR0',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='USR1',func=Pin.BIDIR,do_erc=True),
            Pin(num='18',name='UART-TX',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='UART-RX',func=Pin.BIDIR,do_erc=True)]),
        Part(name='TR-52D',dest=TEMPLATE,tool=SKIDL,keywords='IQRF common transceiver, GMSK modulation',description='IQRF common transceiver, GMSK modulation',ref_prefix='IC',num_units=1,fplist=['IQRF?KON?SIM?01*'],do_erc=True,aliases=['TR-72D', 'DCTR-52D', 'DCTR-72D'],pins=[
            Pin(num='1',name='RA0/AN0/C12IN0',func=Pin.BIDIR,do_erc=True),
            Pin(num='2',name='RC2/Vout',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='Vin',func=Pin.PWRIN,do_erc=True),
            Pin(num='4',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='5',name='RA5/RB4/RC6/AN4/AN11/TX/~SS~/C2OUT/CCP3',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='RC3/SCK/SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='RC4/SDI/SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='RC5/RC7/RX/SDO',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='GND',func=Pin.PWRIN,do_erc=True)]),
        Part(name='XBee_SMT',dest=TEMPLATE,tool=SKIDL,keywords='Digi XBee',description='Digi Xbee SMT RF module',ref_prefix='U',num_units=1,fplist=['Digi*XBee*SMT*'],do_erc=True,pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='VCC',func=Pin.PWRIN,do_erc=True),
            Pin(num='3',name='DIO13/UART_TX',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='DIO14/UART_RX/~CONFIG',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='DIO12',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='RESET/OD_OUT',func=Pin.BIDIR,do_erc=True),
            Pin(num='7',name='DIO10/RSSI/PWM0',func=Pin.BIDIR,do_erc=True),
            Pin(num='8',name='DIO11/PWM1',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='10',name='DIO8/SLEEP_REQUEST',func=Pin.BIDIR,do_erc=True),
            Pin(num='20',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='30',name='DIO3/AD3',func=Pin.BIDIR,do_erc=True),
            Pin(num='11',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='31',name='DIO2/AD2',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='DIO19/SPI_~ATTN',func=Pin.OUTPUT,do_erc=True),
            Pin(num='22',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='32',name='DIO1/AD1',func=Pin.BIDIR,do_erc=True),
            Pin(num='13',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='23',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='33',name='DIO0/AD0',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='DIO18/SPI_CLK',do_erc=True),
            Pin(num='24',name='DIO4',func=Pin.BIDIR,do_erc=True),
            Pin(num='34',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='15',name='DIO17/SPI_~SSEL',do_erc=True),
            Pin(num='25',name='DIO7/~CTS',func=Pin.BIDIR,do_erc=True),
            Pin(num='35',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='16',name='DIO16/SPI_MOSI',do_erc=True),
            Pin(num='26',name='DIO9/ON/~SLEEP',func=Pin.BIDIR,do_erc=True),
            Pin(num='36',name='RF',func=Pin.BIDIR,do_erc=True),
            Pin(num='17',name='DIO15/SPI_MISO',func=Pin.OUTPUT,do_erc=True),
            Pin(num='27',name='VREF',func=Pin.PWRIN,do_erc=True),
            Pin(num='37',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='18',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='28',name='DIO5/ASSOCIATE',func=Pin.BIDIR,do_erc=True),
            Pin(num='19',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='29',name='DIO6/~RTS',func=Pin.BIDIR,do_erc=True)]),
        Part(name='iM880A',dest=TEMPLATE,tool=SKIDL,keywords='IOT LoRa',description='IMST Long Range Radio Module - LoRa Alliance Certified',ref_prefix='U',num_units=1,do_erc=True,aliases=['iM880B'],pins=[
            Pin(num='1',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='2',name='P1-IO/JTCK/SWCLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='3',name='P2-IO/JTMS/SWDIO',func=Pin.BIDIR,do_erc=True),
            Pin(num='4',name='P3-IO/JTDO',func=Pin.BIDIR,do_erc=True),
            Pin(num='5',name='P4-IO/JTDI',func=Pin.BIDIR,do_erc=True),
            Pin(num='6',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='7',name='~RST',do_erc=True),
            Pin(num='8',name='P5-IO/UART-CTS',func=Pin.BIDIR,do_erc=True),
            Pin(num='9',name='P6-IO/UART-RTS',func=Pin.BIDIR,do_erc=True),
            Pin(num='10',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='20',name='P11-IO',func=Pin.BIDIR,do_erc=True),
            Pin(num='30',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='11',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='21',name='P12-IO/I2C-SCL',func=Pin.BIDIR,do_erc=True),
            Pin(num='31',name='RF',func=Pin.BIDIR,do_erc=True),
            Pin(num='12',name='P7-IO/SPI-MISO',func=Pin.BIDIR,do_erc=True),
            Pin(num='22',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='32',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='13',name='P8-IO/SPI-MOSI',func=Pin.BIDIR,do_erc=True),
            Pin(num='23',name='P13-IO/I2C-SDA',func=Pin.BIDIR,do_erc=True),
            Pin(num='14',name='P9-IO/SPI-CLK',func=Pin.BIDIR,do_erc=True),
            Pin(num='24',name='P14-IO/ADC',func=Pin.BIDIR,do_erc=True),
            Pin(num='15',name='P10-IO/SPI-NSS',func=Pin.BIDIR,do_erc=True),
            Pin(num='25',name='P15-IO/WKUP',func=Pin.BIDIR,do_erc=True),
            Pin(num='16',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='26',name='BOOT',do_erc=True),
            Pin(num='17',name='VDD',func=Pin.PWRIN,do_erc=True),
            Pin(num='27',name='GND',func=Pin.PWRIN,do_erc=True),
            Pin(num='18',name='RxD-IO/UART-RX',func=Pin.BIDIR,do_erc=True),
            Pin(num='28',name='NC',func=Pin.NOCONNECT,do_erc=True),
            Pin(num='19',name='TxD-IO/UART-TX',func=Pin.BIDIR,do_erc=True),
            Pin(num='29',name='P17-IO/ADC',func=Pin.BIDIR,do_erc=True)])])